%% Kyoto University, Statistical Machine Learning - Lecture 4 scripts and data
% Marco Cuturi, 2011
set(0,'defaulttextinterpreter','latex');
% Getting the data
% Obtained from http://www.amstat.org/publications/jse/datasets/body.dat
% See http://www.amstat.org/publications/jse/v11n2/datasets.heinz.html

load body.dat
% Body has now 25 columns and 507 observations. For the purpose of this
% lecture, we are only interested in the last 3 measurements: weight (kg)
% height (cm) and gender (1-M , 0-F)

body=body(:,end-2:end);

% compute density for illustration purposes. We will assume this is the
% "real" density.

% indices corresponding to males and females
IndM= (body(:,3)==1);
IndF= (body(:,3)==0);

% ratio of counts of M/F
MF_rat= sum(IndM+1-1)/sum(IndF+1-1);
p_M=sum(IndM+1-1)/size(body,1);


%%%%%%%%%%%%%%%%
% compute the Heights of Males or Females HM,HF, and plot an estimated density.

[HM,XHM]= ksdensity(body(IndM,2));
[HF,XHF]= ksdensity(body(IndF,2));

% to make sure density estimates are computed in the same points.
XH=sort(unique([XHM,XHF]));
XHM=XH;
XHF=XH;
HM= ksdensity(body(IndM,2),XH);
HF= ksdensity(body(IndF,2),XH);


% housekeeping
%close all;
rand('state',0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Hoeffding Inequality Demo
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

FUNCTION=@(x,threshold) .5*sign(x-threshold)+.5;
% compute true risk10
TRUE_RISK= @(threshold) trapz(XH,(FUNCTION(XH,threshold)~=1).*(HM*p_M)+(FUNCTION(XH,threshold)~=0).*(HF*(1-p_M)) );

% compute density estimates to draw random samples.
Distmales=fitdist(body(IndM,2),'kernel');
Distfemales=fitdist(body(IndF,2),'kernel');

% number of samples to test Hoeffding's inequality
N=300;


threshold=165;
TRUE_RISK_=TRUE_RISK(threshold);
EMPIRICAL_RISK=zeros(1,N);
RANDNOISE=rand(1,N);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% PREPARE PLOTS
figure(2);
subplot(1,3,1);
cla reset;
hold on;
plot(XHM,HM*p_M,'b-','LineWidth',2);
plot(XHF,HF*(1-p_M),'r-','LineWidth',2);
legend({'p(X,Y=1)','p(X,Y=0)'})
set(gca,'FontSize',20);
axis tight;


subplot(1,3,2);
cla reset;
hold on;
ylim([.4,.6]);
set(gca,'ytick',[]);
h=vline_(TRUE_RISK_,'b-',['True Risk : ',num2str(TRUE_RISK_)]);
set(h,'LineWidth',2);
xlim([0,1]);
set(gca,'FontSize',20);
hold off;

subplot(1,3,3);
FF= @(x) FUNCTION(x,threshold);
ezplot(FF,[min(XH),max(XH)]);
hhh=title('Considered function $f$');
set(hhh,'Interpreter','latex');
pause();
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




for samples=1:N,
    % number of samples for each testok
    n=30;
    Z=zeros(n,2);
    
    %% PLOTTING stuff
    figure(2);
    subplot(1,3,1);
    cla reset;
    hold on;
    plot(XHM,HM*p_M,'b--','LineWidth',1);
    plot(XHF,HF*(1-p_M),'r--','LineWidth',1);
    set(gca,'FontSize',20);
    axis tight;
    Height=0.015;
    
    for ind=1:n,
        % first decide if sampling from male or female distribution
        if rand()<p_M,
            Z(ind,:)=[random(Distmales),1];
            h=scatter(Z(ind,1),Height*(1+.1*(rand()-.5)),60,'blue','o');
            set(h,'LineWidth',1.5);
        else
            Z(ind,:)=[random(Distfemales),0];
            h=scatter(Z(ind,1),Height*(1+.1*(rand()-.5)),60,'red','d');
            set(h,'LineWidth',1.5);
        end
    end
    hold off;
    if samples<=10,
        pause();
    end
    EMPIRICAL_RISK(samples)=sum(FUNCTION(Z(:,1),threshold)~=Z(:,2))/n;
    
    %% SECOND PLOTTING stuff
    subplot(1,3,2);
    cla reset;
    hold on;
    h=scatter(EMPIRICAL_RISK(1:samples),0.5*ones(1,samples)+.1*(RANDNOISE(1:samples)-.5),120,'black','x');
    set(h,'LineWidth',1);
    ylim([.4,.6]);
    set(gca,'ytick',[]);
    h=vline_(TRUE_RISK_,'b-','True Risk');
    set(h,'LineWidth',2);
    xlim([0,1]);
    set(gca,'FontSize',20);
    hold off;
    %% THIRD PLOTTING STUFF
    if samples>5,
        subplot(1,3,3);
        cla reset;
        hold on;
        [NN,xout]=hist(abs(EMPIRICAL_RISK(1:samples)-TRUE_RISK_),0:0.025:0.6);
        bar(xout,NN/sum(NN));
        axis([0,.6,0,1]);
        hold off;
        title('Frequency of deviations');
        pause(.01);drawnow;
        set(gca,'FontSize',20);
    else
        pause();
    end
end












